#include <avr/io.h>
#include <string.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/sleep.h>

// onewire.h declares pins

// link with options: -Wl,-u,vfprintf -lprintf_flt -lm, and add libs

// Quarz = 20Mhz

#include "i2cmaster.h"
#include "uart.h"
#include "onewire.h"
#include "libsht.h"
#include "ds18x20.h"
#include "clock.h"
#include "rtc.h"
#include "T6963c.h"
#include "drehgeber.h"

#define DEBUG 1
#define MIT_SD_CARD 0

#define TMP175_Address		0b11101010 // A2 floating, A1=A0=1
#define DS3231_Address		0b11010000 // fixe Adresse
#define LC1025_Address		0b10101110 // A2 muss auf VCC liegen (A0=A1=1)
#define CHANNEL_DRUCKSENSOR	7	// AD-Channel fr Drucksensor

#define UART_BAUD_RATE		38400

#define LED_TESTPORT 	PORTD
#define LED_TESTDDR		DDRD
#define LED_TESTPIN		PD7
#define LED_ERRPIN 		PD5



#define MAXSENSORS 6
uint8_t gSensorIDs[MAXSENSORS][OW_ROMCODE_SIZE];	//OW_ROMCODE_SIZE= 0=Family Code; 1-6=SerialNr; 7=CRC

char dat_string[9+9+5+5+5+10*MAXSENSORS+2]; // Datum;Zeit;Temp;Druck;Humi;(SensID;Temp;)*MaxSensor;#

volatile uint8_t has_changed=0;
volatile uint8_t messStart=20;


// ****************************
uint8_t search_sensors(void)
{
	uint8_t i;
	uint8_t id[OW_ROMCODE_SIZE];
	uint8_t diff, nSensors;
	
	uart_puts_P( "\rScanning Bus for DS18X20\r" );
	
	nSensors = 0;
	
	for( diff = OW_SEARCH_FIRST; 
		diff != OW_LAST_DEVICE && nSensors < MAXSENSORS ; )
	{
		DS18X20_find_sensor( &diff, &id[0] );
		
		if( diff == OW_PRESENCE_ERR ) {
			uart_puts_P( "No Sensor found\r" );
			break;
		}
		
		if( diff == OW_DATA_ERR ) {
			uart_puts_P( "Bus Error\r" );
			break;
		}
		
		for (i=0;i<OW_ROMCODE_SIZE;i++)
			gSensorIDs[nSensors][i]=id[i];
		
		nSensors++;
	}
	
	return nSensors;
}

// *******************************************************************    

// ************ main ***********
int main(void)
{


	uint8_t i,j;
	char	buffer[140];
	uint8_t dsSensors;
	uint8_t subzero, cel, cel_frac_bits;
	char buffer_lcd[MAXSENSORS][40]; 		// <-- ********* ACHTUNG ****** 40 Bytes fr eine Zeile
	double temp175=0;
	double Druck=0;
	uint16_t ad_out;
	uint8_t tmp_data1, tmp_data2;
	uint8_t Stunden=0, Minuten=0, Sekunden=0;
	uint8_t Tag=0, Monat=0, Jahr=0;
	uint8_t TempSign=0, Temp=0, TempFrac=0;
	char cTempSign;
	unsigned char error =0;
	unsigned char checksum;
	sht_value humidity;
	sht_value temperature;
	uint8_t val=0, val_neu=0;


	humidity.f=0; humidity.i=0;
	temperature.f=0; temperature.i=0;

	// LEDs auf Ausgang schalten
	LED_TESTDDR |= (1 << LED_TESTPIN) | (1<<LED_ERRPIN);
	LED_TESTPORT |= (1<<LED_TESTPIN);

	
	uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU));
	
	// Drehgeber initialisieren
	encode_init();

	// setup Timer1 for measurement 
	
	//CTC Mode 4
	TCCR1A = (0<<COM1A1) | (0<<COM1A0) | (0<<COM1B1) | (0<<COM1B0) | (0<<WGM11) | (0<<WGM10);
	//CS=101 = prescaler 1024
	TCCR1B = (0<<ICNC1) | (0<<ICES1) | (0<<WGM13) | (1<<WGM12)| (1<<CS12) | (0<<CS11) | (1<<CS10); 
	// Compare register (OCRnx=F_CPU/(Prescaler*gew.Frequenz)-1
	OCR1A = F_CPU/1024-1; 				//je 1s = 19.531,25 
	//OCR0B = (uint8_t)(F_CPU / 64.0 * 1e-3 - 0.5);	// 1ms
	// Intterupt maskieren auf OCIE1A
	TIMSK1 |= (1<<OCIE1A);

	sei();
	
	uart_puts_P("\n+++ Programm Start +++\n");
	_delay_ms(20); 
    

	i2c_init(); // initialize I2C library 

	uart_puts_P("I2C Init ok.\n");


	//Initialize
	//XOR mode, internal CG, Text and Graphic, Blinking Cursor
	T6963cInit(	T6963C_MODE_XOR | T6963C_CG_INTERNALROM,
				T6963C_TEXT_GRAPHIC);
	
	uart_puts_P("Display Init ok.\n");
	
	Start_Clock();

	ow_reset();
	dsSensors = search_sensors();
	DS18X20_read_meas_all_verbose();

	if (UHRSTELLEN)
	{ rtc_init();  // nur bei DS1307, nicht bei DS1337
	  rtc_initClock(SETDATE, SETTIME);
	}

	// Temperatur-Conversion starten
	for ( i=0; i<dsSensors; i++ ) 
	{
		DS18X20_start_meas( DS18X20_POWER_EXTERN, &gSensorIDs[i][0]);
		sprintf(buffer_lcd[i]," ");
	}
	_delay_ms(DS18B20_TCONV_12BIT);  // wahrscheinlich nicht ntig

	while (1)
	{	
		
		if (has_changed)
		{
			
		
		cTempSign=' ';
		if (TempSign) cTempSign='-';


		// *** DS1820 Sensoren anzeigen ***
		for (i=0;i<dsSensors;i++)
		{	
			T6963cPutStringXY(0,i+1,buffer_lcd[i]);
			//uart_puts(buffer_lcd[i]);uart_puts("\n");
		}				
		


		has_changed=0;
	} // if has_changed

	// *** Drehencoder  ***
	if (enc_switch)
	{	
		T6963cPutStringXY_P(0,12, PSTR("Switch gedrueckt"));
		_delay_ms(200);
		enc_switch=0;
	 	
		T6963cClear();
		//Draw a thin dotted Line
		for (i=0; i<200; i++)
		{   // data
			//T6963cSetPixelXY(i+20, i/2);
		}
		// X-Achse zeichnen 
		for (i=0; i<=200; i++) T6963cSetPixelXY(i+20, 112);
		// X-Achse Tickmarks
		for (i=0;i<9;i++) 
		  for (j=0;j<3;j++)
		    T6963cSetPixelXY(i*25+20, 113+j);
		// Y-Achse zeichnen
		for (i=0; i<100;i++) T6963cSetPixelXY(20, i+12);
		// Y-Achse Tickmarks
		for (i=0;i<6;i++) 
		  for (j=0;j<3;j++)
		    T6963cSetPixelXY(20-j, i*20+12);
	}
	else
		T6963cPutStringXY_P(0,12, PSTR("                "));
	
	val += encode_read2();
	if (val_neu > val)
	{ 
	   //sprintf(buffer,"%i\n", val);	  uart_puts(buffer); _delay_ms(30);
       T6963cPutStringXY_P(0,13, PSTR("rechts"));
		_delay_ms(100);		
	   val_neu=val;
	}
	if (val_neu < val)
	{ 
       T6963cPutStringXY_P(0,13, PSTR("links "));
		_delay_ms(100);				
	   val_neu=val;
	}
	if (val_neu==val)
	       T6963cPutStringXY_P(0,13, PSTR("      "));

	// *** Messungen starten ***
	if (messStart>=30) 
	{   		
	
		// ************** DS1820 auslesen ************** 
		for ( i=0; i<dsSensors; i++ ) 
		{
			if ( DS18X20_read_meas( &gSensorIDs[i][0], &subzero,
						&cel, &cel_frac_bits) == DS18X20_OK ) 
			{
				if (subzero)
				  sprintf(buffer, "FC:%X;%X;-%2i.%i", gSensorIDs[i][0], gSensorIDs[i][1], cel, cel_frac_bits);	// gSensorIDs[i][1]= 1. Byte der Seriennummer 				
				else
				  sprintf(buffer, "FC:%X;%X;%2i.%i",  gSensorIDs[i][0], gSensorIDs[i][1], cel, cel_frac_bits);
				//	uart_puts(buffer);
				strcpy(buffer_lcd[i],buffer);
			}
				else uart_puts_P("CRC Error (lost connection?)");																					
		}
		// und neue Conversion starten
		for (i=0;i<dsSensors;i++)
		{	
			// neue Wandlung starten
			DS18X20_start_meas( DS18X20_POWER_EXTERN, &gSensorIDs[i][0]);
		}

	  	messStart=0;
		LED_TESTPORT ^= (1<<LED_TESTPIN);
	}
} // while


	// never reached	
}

ISR( TIMER1_COMPA_vect )				// 1s for measures
{
  
  if (!has_changed) 
  { has_changed=1;
    
  } 

  messStart++;

  LED_TESTPORT ^= (1<<LED_ERRPIN);

}

